@@ -58,17 +58,15 @@ sub new {
     # depending on how $resolver and $rejector are used.
     my $resolver = sub {
         $$value_sr = $_[0];
+        bless $value_sr, _RESOLUTION_CLASS();
 
         # NB: UNIVERSAL::can() is used in order to avoid an eval {}.
         # It is acknowledged that many Perl experts strongly discourage
         # use of this technique.
         if ( UNIVERSAL::can( $$value_sr, 'then' ) ) {
-            return _repromise( $value_sr, \@children, $value_sr );
+            _repromise( $value_sr, \@children, $value_sr );
         }
-
-        bless $value_sr, _RESOLUTION_CLASS();
-
-        if (@children) {
+        elsif (@children) {
             $_->_settle($value_sr) for splice @children;
         }
     };
@@ -173,13 +171,15 @@ sub _repromise {
 #    return (_PENDING_CLASS ne ref $_[0][ _VALUE_SR_IDX ]);
 #}
 
+my ($settle_is_rejection, $self_is_finally);
+
 # This method *only* runs to “settle” a promise.
 sub _settle {
     my ( $self, $final_value_sr ) = @_;
 
     die "$self already settled!" if _PENDING_CLASS ne ref $self->[_VALUE_SR_IDX];
 
-    my $settle_is_rejection = _REJECTION_CLASS eq ref $final_value_sr;
+    $settle_is_rejection = _REJECTION_CLASS eq ref $final_value_sr;
 
     # This has to happen up-front or else we can get spurious
     # unhandled-rejection warnings in asynchronous mode.
@@ -187,7 +187,10 @@ sub _settle {
 
     if ($Promise::ES6::_EVENT) {
         _postpone( sub {
-            $self->_settle_now($final_value_sr, $settle_is_rejection);
+            eval {
+                $self->_settle_now($final_value_sr, $settle_is_rejection);
+                1;
+            } or warn;
         } );
     }
     else {
@@ -198,7 +201,7 @@ sub _settle {
 sub _settle_now {
     my ( $self, $final_value_sr, $settle_is_rejection ) = @_;
 
-    my $self_is_finally = $self->[_IS_FINALLY_IDX];
+    $self_is_finally = $self->[_IS_FINALLY_IDX];
 
     # A promise that new() created won’t have on-settle callbacks,
     # but a promise that came from then/catch/finally will.
@@ -296,7 +299,8 @@ sub _settle_now {
 
         return _repromise( @{$self}[ _VALUE_SR_IDX, _CHILDREN_IDX, _VALUE_SR_IDX, _IS_FINALLY_IDX ] );
     }
-    elsif ( @{ $self->[_CHILDREN_IDX] } ) {
+
+    if ( @{ $self->[_CHILDREN_IDX] } ) {
         $_->_settle( $self->[_VALUE_SR_IDX] ) for splice @{ $self->[_CHILDREN_IDX] };
     }
 
@@ -310,10 +314,10 @@ sub DESTROY {
         warn( ( '=' x 70 ) . "\n" . 'XXXXXX - ' . ref( $_[0] ) . " survived until global destruction; memory leak likely!\n" . ( "=" x 70 ) . "\n" );
     }
 
-    if ( defined $_[0][_VALUE_SR_IDX] ) {
-        my $promise_value_sr = $_[0][_VALUE_SR_IDX];
+    if ( my $promise_value_sr = $_[0][_VALUE_SR_IDX] ) {
         if ( my $value_sr = delete $_UNHANDLED_REJECTIONS{$promise_value_sr} ) {
-            warn "$_[0]: Unhandled rejection: $$value_sr";
+            my $ref = ref $_[0];
+            warn "$ref: Unhandled rejection: $$value_sr";
         }
     }
 }
